@@ -118,7 +118,7 @@ CWLPointerResource::CWLPointerResource(SP<CWlPointer> resource_, SP<CWLSeatResou
 
     m_resource->setSetCursor([this](CWlPointer* r, uint32_t serial, wl_resource* surf, int32_t hotX, int32_t hotY) {
         if (!m_owner) {
-            LOGM(ERR, "Client bug: setCursor when seatClient is already dead");
+            LOGM(Log::ERR, "Client bug: setCursor when seatClient is already dead");
             return;
         }
 
@@ -162,7 +162,7 @@ void CWLPointerResource::sendEnter(SP<CWLSurfaceResource> surface, const Vector2
         return;
 
     if (m_currentSurface) {
-        LOGM(WARN, "requested CWLPointerResource::sendEnter without sendLeave first.");
+        LOGM(Log::WARN, "requested CWLPointerResource::sendEnter without sendLeave first.");
         sendLeave();
     }
 
@@ -218,10 +218,10 @@ void CWLPointerResource::sendButton(uint32_t timeMs, uint32_t button, wl_pointer
         return;
 
     if (state == WL_POINTER_BUTTON_STATE_RELEASED && std::ranges::find(m_pressedButtons, button) == m_pressedButtons.end()) {
-        LOGM(ERR, "sendButton release on a non-pressed button");
+        LOGM(Log::ERR, "sendButton release on a non-pressed button");
         return;
     } else if (state == WL_POINTER_BUTTON_STATE_PRESSED && std::ranges::find(m_pressedButtons, button) != m_pressedButtons.end()) {
-        LOGM(ERR, "sendButton press on a non-pressed button");
+        LOGM(Log::ERR, "sendButton press on a non-pressed button");
         return;
     }
 
@@ -328,7 +328,7 @@ CWLKeyboardResource::CWLKeyboardResource(SP<CWlKeyboard> resource_, SP<CWLSeatRe
     m_resource->setOnDestroy([this](CWlKeyboard* r) { PROTO::seat->destroyResource(this); });
 
     if (!g_pSeatManager->m_keyboard) {
-        LOGM(ERR, "No keyboard on bound wl_keyboard??");
+        LOGM(Log::ERR, "No keyboard on bound wl_keyboard??");
         return;
     }
 
@@ -380,7 +380,7 @@ void CWLKeyboardResource::sendEnter(SP<CWLSurfaceResource> surface, wl_array* ke
         return;
 
     if (m_currentSurface) {
-        LOGM(WARN, "requested CWLKeyboardResource::sendEnter without sendLeave first.");
+        LOGM(Log::WARN, "requested CWLKeyboardResource::sendEnter without sendLeave first.");
         sendLeave();
     }
 
@@ -531,7 +531,7 @@ void CWLSeatProtocol::bindManager(wl_client* client, void* data, uint32_t ver, u
 
     RESOURCE->m_self = RESOURCE;
 
-    LOGM(LOG, "New seat resource bound at {:x}", (uintptr_t)RESOURCE.get());
+    LOGM(Log::DEBUG, "New seat resource bound at {:x}", (uintptr_t)RESOURCE.get());
 
     m_events.newSeatResource.emit(RESOURCE);
 }
